#include "Zeolite.h"
#include "zmaptile.h"

CzMapTile::CzMapTile(void) {

	m_pTile = NULL;
	m_hMap = NULL;

	// direct interface variables
	m_DirectInterfaceFlag = false;
	m_TileSize = 0;
	m_PixelMemSize = 0;
	m_pMem = NULL;
	m_Contiguous = false;
}

CzMapTile::~CzMapTile(void) {
	Release();
}

bool CzMapTile::GetMapTile(ZMAP hMap, long ti, long tj) {

	if(m_pTile) {
		Release();
	}

	void* pTile = zmap_GetMosaicTile(hMap, ti, tj);
	if(!pTile) {
		zeoReportError("CzMapTile::GetMapTile error:\r\n - cannot get mosaic tile");
		return false;
	}

	if(!Attach(pTile, hMap)) {
		zeoReportError("CzMapTile::GetMapTile error:\r\n - cannot attach mosaic tile");
		return false;
	}

	return true;
}

bool CzMapTile::Attach(void* pTile, ZMAP hMap) {

	// if already attached, make sure we close the prev direct interface or lese risk leaking the scanline table
	if(m_pTile && m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}
	
	if(!zmap_tile_SetLock(pTile, true)) {
		zeoReportError("CzMapTile::Attach error:\r\n - cannot lock tile");
		return false;
	}

	m_pTile = pTile;
	m_hMap = hMap;

	return true;
}
void CzMapTile::Release() {
	
	if(!m_pTile) {
		return;
	}

	if(m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}

	zmap_tile_SetLock(m_pTile, false);

	m_pTile = NULL;
	m_hMap = NULL;
}


bool CzMapTile::GetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(m_Contiguous) {
			int k = x + y * m_TileSize;
			memcpy(pValue, (char*)m_pMem + k * m_PixelMemSize, m_PixelMemSize);
			return true;
		}
		
		memcpy(pValue, ((char**)m_pMem)[y] + x * m_PixelMemSize, m_PixelMemSize);
		return true;
	}

	return zmap_tile_GetPixel(m_pTile, x, y, pValue);
}
bool CzMapTile::SetPixel(long x, long y, void* pValue) {

	if(m_DirectInterfaceFlag) {
		if(m_Contiguous) {
			int k = x + y * m_TileSize;
			memcpy((char*)m_pMem + k * m_PixelMemSize, pValue, m_PixelMemSize);
			return true;
		}

		memcpy( ((char**)m_pMem)[y] + x * m_PixelMemSize, pValue, m_PixelMemSize);
		return true;
	}

	return zmap_tile_SetPixel(m_pTile, x, y, pValue);
}

void* CzMapTile::GetAt(long x, long y) {

	long TileSize = zmap_GetTileSize(m_hMap);
	if(!TileSize)	return NULL;

	if(x<0 || x>=TileSize)	return NULL;
	if(y<0 || y>=TileSize)	return NULL;


	/*
	void* pMapData = NULL;
	if(m_DirectInterfaceFlag)
			pMapData = m_pMem;
	else 	pMapData = zmap_tile_GetDataPtr(m_pTile);
	if(!pMapData) return NULL;
	
	__int64 k = (__int64)x + (__int64)y * (__int64)TileSize;
	long PxMemSize = zmap_GetPixelSize(m_hMap);

	return (void*)((char*)pMapData + (__int64)PxMemSize * k);

	*/
	
	void* pLine = NULL;
	if(!m_DirectInterfaceFlag) {
		pLine = zmap_tile_GetScanlinePtr(m_pTile, y);
	} else {
		if(m_Contiguous) {
			pLine = (void*)( (char*)m_pMem + m_PixelMemSize * TileSize * y);
		} else {
			pLine = ((void**)m_pMem)[y];
		}
	}
	
	if(!pLine)	return NULL;
	
	return (char*)pLine + x * m_PixelMemSize;

}

bool CzMapTile::IsContiguous() {
	if(!m_pTile) {
		return false;
	}

	return zmap_tile_IsContiguous(m_pTile);
}
bool CzMapTile::MakeContiguous() {
	if(!m_pTile) {
		return false;
	}

	if(m_DirectInterfaceFlag) {
		CloseDirectInterface();
	}

	return zmap_tile_MakeContiguous(m_pTile);
}
bool CzMapTile::AsContiguousMap(CzMap& TileMap) {

	if(!m_pTile) {
		zeoReportError("CzMapTile::AsContiguousMap error:\r\n - no tile attached");
		return false;
	}

	bool OldDirectInterface = m_DirectInterfaceFlag;
	if(!m_DirectInterfaceFlag) {
		if(!OpenDirectInterface()) {
			zeoReportError("CzMapTile::AsContiguousMap error:\r\n - cannot open direct interface");
			return false;
		}
	}

	if(TileMap.nPixels()) {
		if(TileMap.nx()!=m_TileSize || TileMap.ny()!=m_TileSize || TileMap.GetMapType()!=zmap_GetMapType(m_hMap)) {
			TileMap.Free();
		} else
		if(TileMap.GetAllocMode()!=MAPALLOC_RAM || !TileMap.IsContiguous()) {
			TileMap.Free();
		}
	}
	if(!TileMap.nPixels()) {
		if(!TileMap.Init2(m_TileSize, m_TileSize, zmap_GetMapType(m_hMap), zmap_GetHorizScale(m_hMap), false, MAPALLOC_RAM_CONTIGUOUS)) {

			zeoReportError("CzMapTile::AsContiguousMap error:\r\n - cannot initialise tile map");


			// cleanup
			if(!OldDirectInterface)	CloseDirectInterface();
			return false;
		}
	}


	// and open a direct interface to mem
	if(!TileMap.OpenDirectInterface()) {

		zeoReportError("CzMapTile::AsContiguousMap error:\r\n - cannot open direct interface with tile map");

		// cleanup
		if(!OldDirectInterface)	CloseDirectInterface();
		return false;
	}

	void* pTileMapData = TileMap.GetDataPtr();
	if(!pTileMapData) {

		zeoReportError("CzMapTile::AsContiguousMap error:\r\n - invalid pointer to tile map data");

		// cleanup
		if(!OldDirectInterface)	CloseDirectInterface();
		return false;
	}

	long LineLen = m_PixelMemSize * m_TileSize;

	for(int j=0; j<m_TileSize; j++) {
		void* pSrc = GetScanlinePtr(j);
		if(!pSrc) {
			zeoReportError("CzMapTile::AsContiguousMap error:\r\n - cannot get source line ptr");

			// cleanup
			if(!OldDirectInterface)	CloseDirectInterface();
			return false;
		}

		void* pDest = (void*)( (char*)pTileMapData + j * LineLen);
		
		memcpy(pDest, pSrc, LineLen);
	}
	
	// cleanup
	if(!OldDirectInterface)	CloseDirectInterface();

	return true;
}

// direct interface
bool CzMapTile::OpenDirectInterface() {

	if(m_DirectInterfaceFlag) {
		zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - already open");
		return false;
	}

	m_TileSize = zmap_GetTileSize(m_hMap);
	if(!m_TileSize) {
		zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - invalid tile size");
		return false;
	}

	m_PixelMemSize = zmap_GetPixelSize(m_hMap);
	if(!m_PixelMemSize) {
		zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - invalid pixel mem size");
		return false;
	}

	m_Contiguous = zmap_tile_IsContiguous(m_pTile);
	if(m_Contiguous) {
		m_pMem = zmap_tile_GetDataPtr(m_pTile);
		if(!m_pMem) {
			zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - cannot get tile data handle");
			return false;
		}
	} else {
		// alloc line list
		m_pMem = (void*)zmap_tile_GetScanlineArray(m_pTile);
		if(!m_pMem) {
			zeoReportError("CzMapTile::OpenDirectInterface error:\r\n - cannot allocate scanline array");
			return false;
		}
	}

	m_DirectInterfaceFlag = true;

	return true;
}
void CzMapTile::CloseDirectInterface() {

	if(!m_DirectInterfaceFlag) {
		return;
	}

	m_pMem = NULL;

	m_DirectInterfaceFlag = false;
}

bool CzMapTile::MarkChanged() {
	return zmap_tile_MarkChanged(m_pTile);
}

void* CzMapTile::GetScanlinePtr(long j) {
	return zmap_tile_GetScanlinePtr(m_pTile, j);
}