#include <zeolite.h>
#include "zMeshIO.h"
#include "zmesh.h"
#include <zfile.h>
#include <zstr.h>
#include <zfunc.h>

bool zmesh_Load(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_Load error:\r\n - invalid mesh handle");
		return false;
	}
	
	CzStr FileExt;
	zfile_GetExt(lpMeshFile, FileExt);
	if(0==FileExt.CompareNoCase("zmesh")) {
		if(!zmesh_LoadZMESH(pMesh, lpMeshFile, Flags, hOptions)) {
			zeoReportError("zmesh_Load error:\r\n - call to zmesh_LoadZMESH failed");
			return false;
		}
	} else
	if(0==FileExt.CompareNoCase("obj")) {
		if(!zmesh_LoadOBJ(pMesh, lpMeshFile, Flags, hOptions)) {
			zeoReportError("zmesh_Load error:\r\n - call to zmesh_LoadOBJ failed");
			return false;
		}
	} else
	if(0==FileExt.CompareNoCase("s3o")) {
		if(!zmesh_LoadS3O(pMesh, lpMeshFile, Flags, hOptions)) {
			zeoReportError("zmesh_Load error:\r\n - call to zmesh_LoadS3O failed");
			return false;
		}
	} else {
		zeoReportError("zmesh_Load error:\r\n - unrecognised file type");
		return false;
	}

	// now convert relative material paths to absolute
	CzStr zDirName;
	if(!zfile_GetDir(lpMeshFile, zDirName)) {
		zeoReportError("zmesh_Load error:\r\n - cannot get directory name");
		return false;
	}
	if(!zmesh_LoadMaterialPaths(pMesh, (char*)zDirName)) {
		zeoReportError("zmesh_Load error:\r\n - cannot load material paths");
		return false;
	}

	// and check mesh contents
	if(!pMesh->Validate()) {
		zeoWriteToLog("zmesh_Load error:\r\n - mesh is invalid");
		return false;
	}

	return true;
}
bool zmesh_Save(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_Save error:\r\n - invalid mesh handle");
		return false;
	}

	// save file
	CzStr FileExt;
	zfile_GetExt(lpMeshFile, FileExt);
	if(0==FileExt.CompareNoCase("zmesh")) {
		return zmesh_SaveZMESH(pMesh, lpMeshFile, Flags, hOptions);
	} else 
	if(0==FileExt.CompareNoCase("obj")) {
		return zmesh_SaveOBJ(pMesh, lpMeshFile, Flags, hOptions);
	} else 
	if(0==FileExt.CompareNoCase("3ds")) {
		return zmesh_Save3DS(pMesh, lpMeshFile, Flags, hOptions);
	} else 
	if(0==FileExt.CompareNoCase("dae")) {
		return zmesh_SaveDAE(pMesh, lpMeshFile, Flags, hOptions);
	} else 
	if(0==FileExt.CompareNoCase("x")) {
		return zmesh_SaveX(pMesh, lpMeshFile, Flags, hOptions);
	} //else
	//if(0==FileExt.CompareNoCase("s3o")) { // not yet implemented
	//	return zmesh_SaveS3O(&m2, lpMeshFile, Flags, hOptions);
	//}

	// if you got here, the file format was unsupported
	zeoReportError("zmesh_Save error:\r\n - urecognised file type");
	return false;
}

bool zmesh_LoadZMESH(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {


	if(!pMesh) {
		zeoReportError("zmesh_LoadZMESH error:\r\n - invalid mesh handle");
		return false;
	}

	if(!pMesh->GetZVAR()) {
		if(!pMesh->Create()) {
			zeoReportError("zmesh_LoadZMESH error:\r\n - cannot create mesh");
			return false;
		}
	}



	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.ZMESH.LoadMesh")) {
		zeoReportError("zmesh_LoadZMESH error:\r\n - cannot get 'file.ZMESH.LoadMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_LoadZMESH error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);


	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_LoadZMESH error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_LoadZMESH error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_LoadZMESH error:\r\n - call to 'file.ZMESH.LoadMesh' returned false");
		return false;
	}



	if(!pMesh->LoadHandles()) {
		zeoReportError("zmesh_LoadZMESH error:\r\n - cannot reaload data handles (mesh is invalid)");
		return false;
	}

	return true;
}

 
bool zmesh_LoadOBJ(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {
	
	if(!pMesh) {
		zeoReportError("zmesh_LoadOBJ error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		if(!pMesh->Create()) {
			zeoReportError("zmesh_LoadOBJ error:\r\n - cannot create mesh");
			return false;
		}
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.OBJ.LoadMesh")) {
		zeoReportError("zmesh_LoadOBJ error:\r\n - cannot get 'file.OBJ.LoadMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_LoadOBJ error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);


	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_LoadOBJ error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_LoadOBJ error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_LoadOBJ error:\r\n - call to 'file.OBJ.LoadMesh' returned false");
		return false;
	}

	if(!pMesh->LoadHandles()) {
		zeoReportError("zmesh_LoadOBJ error:\r\n - cannot reaload data handles (mesh is invalid)");
		return false;
	}

	return true;
}

bool zmesh_LoadS3O(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {
	
	if(!pMesh) {
		zeoReportError("zmesh_LoadS3O error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		if(!pMesh->Create()) {
			zeoReportError("zmesh_LoadS3O error:\r\n - cannot create mesh");
			return false;
		}
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.S3O.LoadMesh")) {
		zeoReportError("zmesh_LoadS3O error:\r\n - cannot get 'file.S3O.LoadMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_LoadS3O error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);


	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_LoadS3O error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_LoadS3O error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_LoadS3O error:\r\n - call to file.S3O.LoadMesh returned false");
		return false;
	}

	return true;
}



bool zmesh_SaveZMESH(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_SaveZMESH error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		zeoReportError("zmesh_SaveZMESH error:\r\n - mesh is not initialised");
		return false;
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.ZMESH.SaveMesh")) {
		zeoReportError("zmesh_SaveZMESH error:\r\n - cannot get 'file.ZMESH.SaveMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_SaveZMESH error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);

	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_SaveZMESH error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_SaveZMESH error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_SaveZMESH error:\r\n - call to 'file.ZMESH.SaveMesh' returned false");
		return false;
	}

	return true;
}



bool zmesh_SaveOBJ(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_SaveOBJ error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		zeoReportError("zmesh_SaveOBJ error:\r\n - mesh is not initialised");
		return false;
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.OBJ.SaveMesh")) {
		zeoReportError("zmesh_SaveOBJ error:\r\n - cannot get 'file.OBJ.SaveMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_SaveOBJ error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);

	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_SaveOBJ error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_SaveOBJ error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_SaveOBJ error:\r\n - call to 'file.OBJ.SaveMesh' returned false");
		return false;
	}

	return true;
}

bool zmesh_Save3DS(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_Save3DS error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		zeoReportError("zmesh_Save3DS error:\r\n - mesh is not initialised");
		return false;
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.3DS.SaveMesh")) {
		zeoReportError("zmesh_Save3DS error:\r\n - cannot get 'file.3DS.SaveMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_Save3DS error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);

	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_Save3DS error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_Save3DS error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_Save3DS error:\r\n - call to 'file.3DS.SaveMesh' returned false");
		return false;
	}

	return true;
}

bool zmesh_SaveX(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_SaveX error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		zeoReportError("zmesh_SaveX error:\r\n - mesh is not initialised");
		return false;
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.X.SaveMesh")) {
		zeoReportError("zmesh_SaveX error:\r\n - cannot get 'file.X.SaveMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_SaveX error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);

	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_SaveX error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_SaveX error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_SaveX error:\r\n - call to 'file.X.SaveMesh' returned false");
		return false;
	}

	return true;
}


bool zmesh_SaveDAE(CzMesh* pMesh, const char* lpMeshFile, unsigned long Flags, ZLIST hOptions) {

	if(!pMesh) {
		zeoReportError("zmesh_SaveDAE error:\r\n - invalid mesh handle");
		return false;
	}
	if(!pMesh->GetZVAR()) {
		zeoReportError("zmesh_SaveDAE error:\r\n - mesh is not initialised");
		return false;
	}


	//
	// get function, set arg and execute
	//
	CzFunc f;
	if(!f.GetFunc("file.DAE.SaveMesh")) {
		zeoReportError("zmesh_SaveDAE error:\r\n - cannot get 'file.DAE.SaveMesh' function");
		return false;
	}


	int argid = 0;
	if(!zstr_SetText(f.GetArgI(argid++), lpMeshFile)) {
		zeoReportError("zmesh_SaveDAE error:\r\n - cannot set function arg");
		return false;
	}
	ZVAR hMesh = pMesh->GetZVAR();
	f.SetArgValueI(argid++, VarID_hvar, &hMesh);
	f.SetArgValueI(argid++, VarID_uint, &Flags);
	f.SetArgValueI(argid++, VarID_hvar, &hOptions);

	if(!f.Execute2(NULL, NULL)) {
		zeoReportError("zmesh_SaveDAE error:\r\n - function call failed");
		return false;
	}

	//
	// retrieve result
	//
	bool rval = false;
	if(!f.m_rval.GetValue(VarID_bool, &rval)) {
		zeoReportError("zmesh_SaveDAE error:\r\n - cannot get rval");
		return false;
	}
	if(!rval) {
		// function failed
		zeoReportError("zmesh_SaveDAE error:\r\n - call to 'file.DAE.SaveMesh' returned false");
		return false;
	}

	return true;
}


bool zmesh_LoadMaterialPathFile(ZSTR hImagePath, const char* lpDirName) {
	// convert a path name for an image from relative to absolute

	CzStr s;
	if(!s.Attach(hImagePath)) {
		zeoReportError("zmesh_LoadMaterialPathFile error:\r\n - invalid string handle");
		return false;
	}
	if(!s.GetLength()) {
		// no file name, so return true
		return true;
	}

	// if relative, make positive my prepending dir name
	if(!zfile_IsAbsolutePath((char*)s)) {
		CzStr s2;
		s2.SetText(lpDirName);
		s2.Append("\\");
		s2.Append((char*)s);

		s.SetText((char*)s2);
	}

	// finally, check the file exists
	if(!zfile_FileExists((char*)s)) {
		zeoReportError("zmesh_LoadMaterialPathFile error:\r\n - file not found");
		return false;
	}

	return true;
}

bool zmesh_LoadMaterialPaths(CzMesh* pMesh, const char* lpDirName) {
	// converts path names to images in materials from relative to absolute paths after loading

	if(!pMesh) {
		zeoReportError("zmesh_LoadMaterialPaths error:\r\n - invalid mesh handle");
		return false;
	}

	for(int i=0; i<pMesh->nMaterials(); i++) {
		CzMeshMtl mtl;
		if(!pMesh->GetMaterial(i, mtl)) {
			zeoReportError("zmesh_LoadMaterialPaths error:\r\n - cannot get material");
			return false;
		}

		if(!zmesh_LoadMaterialPathFile(mtl.GetAmbImageFile(), lpDirName)) {
			zeoReportError("zmesh_LoadMaterialPaths error:\r\n - cannot load ambient material path name");
			return false;
		}
		if(!zmesh_LoadMaterialPathFile(mtl.GetDiffImageFile(), lpDirName)) {
			zeoReportError("zmesh_LoadMaterialPaths error:\r\n - cannot load diffuse material path name");
			return false;
		}
		if(!zmesh_LoadMaterialPathFile(mtl.GetSpecImageFile(), lpDirName)) {
			zeoReportError("zmesh_LoadMaterialPaths error:\r\n - cannot load specular material path name");
			return false;
		}
	}

	for(int i=0; i<pMesh->nSubMeshes(); i++) {
		CzMesh m2;
		if(!pMesh->GetSubMesh(i, m2)) {
			zeoReportError("zmesh_LoadMaterialPaths error:\r\n - cannot get submesh");
			return false;
		}
		if(!zmesh_LoadMaterialPaths(&m2, lpDirName)) {
			zeoReportError("zmesh_LoadMaterialPaths error:\r\n - recursive call failed for submesh");
			return false;
		}
	}

	return true;
}


bool zmesh_SaveMaterialPathFile(ZSTR hImagePath, const char* lpDirName, bool CopyImages) {
	
	CzStr s;
	if(!s.Attach(hImagePath)) {
		zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - invalid string handle");
		return false;
	}
	if(!s.GetLength()) {
		// no file name; return quietly
		return true;
	}

	if(!zfile_IsAbsolutePath((char*)s)) {
		// paths in meshes should not be relative, but if they are, we'll try to find it
		 
		// does the file exist at the target destination (if not, we don't know where it is!)
		CzStr s2;
		s2.SetText(lpDirName);
		s2.Append("\\");
		s2.Append((char*)s);
		if(!zfile_FileExists((char*)s2)) {
			zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - image file cannot be found");
			return false;
		}

	} else {
		// an absolute path, so strip lpDirName and copy image file

		// first, check the old file exists
		if(!zfile_FileExists((char*)s)) {
			zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - image file does not exist");
			return false;
		}

		// get its dir and short filename
		CzStr zOldDir;
		if(!zfile_GetDir((char*)s, zOldDir)) {
			zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - cannot get material image path name");
			return false;
		}
		CzStr zShortName;
		if(!zfile_TrimDir((char*)s, zShortName)) {
			zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - cannot get material image directory name");
			return false;
		}

		CzStr zRelPath;
		if(zfile_TrimPath((char*)s, lpDirName, zRelPath)) {
			// the old path is absolute, and within the target directory (or could be made relative)

			if(CopyImages) {

				// format the new target path
				CzStr zTarget;
				zTarget.SetText(lpDirName);
				zTarget.Append("\\");
				zTarget.Append((char*)zRelPath);

				if(0==s.CompareNoCase((char*)zTarget)) {
					// same path; we already checked that the file exists, so do nothing more
				} else {
					// file is to be moved to new folder, so copy
					if(!zfile_CopyFile((char*)s, (char*)zTarget, NULL)) {
						zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - cannot copy material image");
						return false;
					}
				}
			}

			// and update with the new relative path
			if(!s.CopyValue(zRelPath)) {
				zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - cannot copy path name");
				return false;
			}
		} else {
			// the old path is absolute, but outside the target directory (and cannot be made relative).
			if(!CopyImages) {
				// if we're not copying, do nothing (leave path as absolute)
			} else {
				// move image to new path

				// format the new target path
				CzStr zTarget;
				zTarget.SetText(lpDirName);
				zTarget.Append("\\");
				zTarget.Append((char*)zShortName);

				// file is to be moved to new folder, so copy
				if(!zfile_CopyFile((char*)s, (char*)zTarget, NULL)) {
					zeoReportError("zmesh_SaveMaterialPathFile error:\r\n - cannot copy material image");
					return false;
				}
			}
		}
	}

	return true;
}

bool zmesh_SaveMaterialPaths(CzMesh* pMesh, const char* lpDirName, bool CopyImages) {
	// converts path names to images in materials from absolute to relative paths before saving

	if(!pMesh) {
		zeoReportError("zmesh_SaveMaterialPaths error:\r\n - invalid mesh handle");
		return false;
	}

	for(int i=0; i<pMesh->nMaterials(); i++) {
		CzMeshMtl mtl;
		if(!pMesh->GetMaterial(i, mtl)) {
			zeoReportError("zmesh_SaveMaterialPaths error:\r\n - cannot get material");
			return false;
		}

		if(!zmesh_SaveMaterialPathFile(mtl.GetAmbImageFile(), lpDirName, CopyImages)) {
			zeoReportError("zmesh_SaveMaterialPaths error:\r\n - cannot save material ambient image path");
			return false;
		}
		if(!zmesh_SaveMaterialPathFile(mtl.GetDiffImageFile(), lpDirName, CopyImages)) {
			zeoReportError("zmesh_SaveMaterialPaths error:\r\n - cannot save material diffuse image path");
			return false;
		}
		if(!zmesh_SaveMaterialPathFile(mtl.GetSpecImageFile(), lpDirName, CopyImages)) {
			zeoReportError("zmesh_SaveMaterialPaths error:\r\n - cannot save material specular image path");
			return false;
		}
	}

	for(int i=0; i<pMesh->nSubMeshes(); i++) {
		CzMesh m2;
		if(!pMesh->GetSubMesh(i, m2)) {
			zeoReportError("zmesh_SaveMaterialPaths error:\r\n - cannot get submesh");
			return false;
		}
		if(!zmesh_SaveMaterialPaths(&m2, lpDirName, CopyImages)) {
			zeoReportError("zmesh_SaveMaterialPaths error:\r\n - recursive call failed for submesh");
			return false;
		}
	}


	return true;
}